#!/usr/dist/bin/wishx -f

#
# mail.tcl
# Copyright (C) 1995 by John Heidemann
# This program is released under the Gnu Public License, version 2.
# Comments to <johnh@isi.edu>.
#
# TODOs: should make the mailer "reset"able like xbiff.
#

# random in Tcl---courtesy of Don Libes, slightly hacked since then.
proc random {args} {
	global _ran

	if [llength $args]>1 {
		set _ran [lindex $args 1]
	} else {
		set period 233280
		if [info exists _ran] {
			set _ran [expr ($_ran*9301 + 49297) % $period]
		} else {
			set _ran [expr [clock seconds] % $period]
		}
		return [expr int($args*($_ran/double($period)))]
	}
}

proc mail_default_box {} {
	global env
	# env(LOGNAME) is not defined when running from rsh/xon.
	if [info exists env(LOGNAME)] {
		return "/var/spool/mail/$env(LOGNAME)"
	} else {
		return "/var/spool/mail/[exec whoami]"
	}
}

#
# policies:
#

# Policy zero_length is good if you never leave mail in your box.
proc mail_zero_length {} {
	global rdb
	if [file exists $rdb(mail.box)] {
		if [file size $rdb(mail.box)] {
			return 1
		} else {
			return 0
		}
	} else {
		return 0
	}
}

#
# Policy xbiff is the xbiff policy:
# Now check for changes.  If reset is set then we want to pretent that
# there is no mail.  If the mailbox is empty then we want to turn off
# the flag.  Otherwise if the mailbox has changed size then we want to
# put the flag up, unless the mailbox has been read since the last 
# write.
#
# The cases are:
#    o  forced reset by user                        DOWN
#    o  no mailbox or empty (zero-sized) mailbox    DOWN
#    o  if read after most recent write 	    DOWN
#    o  same size as last time                      no change
#    o  bigger than last time                       UP
#    o  smaller than last time but non-zero         UP
#
# The last two cases can be expressed as different from last
# time and non-zero.
proc mail_xbiff {} {
	global rdb
	if {[file exists $rdb(mail.box)] == 0} {
		return 0
	} elseif {[file size $rdb(mail.box)] == 0} {
		return 0
	} elseif {[file atime $rdb(mail.box)] > [file mtime $rdb(mail.box)]} {
		# read sinze last write
		return 1
	} else {
		return 0
	}
}

proc mail_icon_notify {status} {
	if {$status} {
		monitor_bell
		.mail conf -fg $rdb(bg) -bg $rdb(fg)
	} else {
		.mail conf -fg $rdb(fg) -bg $rdb(bg)
	}
}

proc mail_eye_notify {status} {
	global rdb
	if {$status} {
		monitor_bell
		.menu conf -fg $rdb(bg) -bg $rdb(fg)
		after 4900 mail_eye_blink
	} else {
		.menu conf -fg $rdb(fg) -bg $rdb(bg)
	}
}

proc mail_eye_blink {} {
	global mail_status rdb mail_alt_status
	if {$mail_status} {
		if {$mail_alt_status} {
			.menu config -bitmap $rdb(menu.alticon)
			set mail_alt_status 0
		} else {
			.menu config -bitmap $rdb(menu.icon)
			set mail_alt_status 1
		}
		after [expr 2000+[random 6000]] mail_eye_blink
	} else {
		.menu config -bitmap $rdb(menu.icon)
	}
}

proc mail_update {} {
	global rdb monitor_module mail_status mail_alt_status
	if {$monitor_module(mail)} {
		set new_mail_status [$rdb(mail.policy)]
		if { $mail_status != $new_mail_status } {
			if {$new_mail_status != 0} {
				set mail_alt_status 0
				$rdb(mail.notify) 1
			} else {
				$rdb(mail.notify) 0
			}
			set mail_status $new_mail_status
		}
		after 59000 mail_update
	}
}

proc mail_start {} {
	global rdb mail_status env
	set_rdb_default mail.policy mail_xbiff
	set_rdb_default mail.notify mail_icon_notify
	set_rdb_default mail.box [mail_default_box]
	set mail_status -1
	set_rdb_default mail.row 1
	if { $rdb(mail.notify) == "mail_icon_notify" } {
		mk_rows $rdb(mail.row)
		label .mail -bitmap "@$rdb(moduledir)/little_snail.xbm" -fg $rdb(fg) -bg $rdb(bg)
		pack .mail -side $rdb(side) -in ".row$rdb(mail.row)"
	}
	mail_update
}

proc mail_stop {} {
	destroy .mail
}
